﻿//
//TraxsterPlus2
//SJJ Embedded Micro Solutions
//Copyright (c) 2004 - 2010 SJJ Micro Solutions, LLC. All Rights Reserved
//

using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using System.Threading;
using Microsoft.SPOT.Hardware.SJJ;
using Microsoft.SPOT.Hardware.SJJ.SPI_LCD_Driver;
using System.IO.Ports;

namespace TraxsterPlus2
{
    public class Program
    {
        public static void Main()
        {
            Debug.Print(Resources.GetString(Resources.StringResources.String1));
            App myApp = new App();
            myApp.Run();
        }

        public class App
        {
            //configure the SPI port, the LCDDriver, the Serial port and the Green LED port.
            static SPI.Configuration mySPIPortSettings = new SPI.Configuration(Pins.GPIO_NONE, false, 0, 0, false, true, 50, SPI.SPI_module.SPI1);
            SPI mySPIPort = new SPI(mySPIPortSettings);
            SPI_LCDDriver myLCDDriver = new SPI_LCDDriver();

            public SerialPort COM2Port = new SerialPort(SerialPorts.COM2, (int)BaudRates.Baud19200, Parity.None, 8);

            OutputPort myGreenLED = new OutputPort(Pins.GREEN_LED, false);
            
            //Setup the Serializer commands
            public static String sForward = "mogo 1:10 2:10\r";
            public static String sReverse = "mogo 1:-10 2:-10\r";
            public static String sRight = "mogo 1:12 2:-12\r";
            public static String sLeft = "mogo 1:-12 2:12\r";
            public byte[] boutForward = new byte[sForward.Length];
            public byte[] boutReverse = new byte[sReverse.Length];
            public byte[] boutRight = new byte[sRight.Length];
            public byte[] boutLeft = new byte[sLeft.Length];

            public static String sSTOP = "stop\r";
            public static String sSensor = "sensor 0\r";
            public static String sServo_m45 = "servo 1:-45\r";
            public static String sServo_C = "servo 1:0\r";
            public static String sServo_p45 = "servo 1:45\r";
            public byte[] boutStop = new byte[sSTOP.Length];
            public byte[] boutSensor = new byte[sSensor.Length];
            public byte[] boutServo_m45 = new byte[sServo_m45.Length];
            public byte[] boutServo_C = new byte[sServo_C.Length];
            public byte[] boutServo_p45 = new byte[sServo_p45.Length];

            public static String tempSensor = "sensor 4\r";
            public byte[] boutTempSensor = new byte[tempSensor.Length];

            public int ReverseCheck = 0;

            public byte[] binSensorArray = new byte[16]; //256 was an arbitraty number. it could have been 8 or 512


            public void Run()
            {
                //Open the COM2 port
                COM2Port.ReadTimeout = 10;
                COM2Port.Open();

                //Encode the Serializer commands to byte arrays
                System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
                boutForward = Encoding.GetBytes(sForward);
                boutReverse = Encoding.GetBytes(sReverse);
                boutRight = Encoding.GetBytes(sRight);
                boutLeft = Encoding.GetBytes(sLeft);
                boutStop = Encoding.GetBytes(sSTOP);
                boutSensor = Encoding.GetBytes(sSensor);
                boutServo_m45 = Encoding.GetBytes(sServo_m45);
                boutServo_C = Encoding.GetBytes(sServo_C);
                boutServo_p45 = Encoding.GetBytes(sServo_p45);

                //Ready to go!
                mySPIPort.Write(myLCDDriver.LCDClear());
                Thread.Sleep(6);
                mySPIPort.Write(myLCDDriver.LCDLine1("Ready"));
                Thread.Sleep(6);

                //This is the main While-loop that keeps the robot moving forward
                while (true)
                {
                    COM2Port.Write(boutForward, 0, boutForward.Length);
                    Thread.Sleep(100);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);

                    mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Left"));
                    COM2Port.Write(boutServo_m45, 0, boutServo_m45.Length);
                    Thread.Sleep(300);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    if (SensorIR())
                    {

                        Obstruction();

                    }

                    COM2Port.Write(boutForward, 0, boutForward.Length);
                    Thread.Sleep(100);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);

                    mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Center"));
                    COM2Port.Write(boutServo_C, 0, boutServo_C.Length);
                    Thread.Sleep(300);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    if (SensorIR())
                    {
                        Obstruction();

                    }

                    COM2Port.Write(boutForward, 0, boutForward.Length);
                    Thread.Sleep(100);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);

                    mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Right"));
                    COM2Port.Write(boutServo_p45, 0, boutServo_p45.Length);
                    Thread.Sleep(300);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    if (SensorIR())
                    {

                        Obstruction();

                    }
                    Thread.Sleep(50);

                }

                
            }

            //An obstruction was found, stop the motors and scan to find
            // a clear path and keep moving
            public void Obstruction()
            {
                
                myGreenLED.Write(true);
                mySPIPort.Write(myLCDDriver.LCDLine1("Obstruction Found"));
                Thread.Sleep(6);
                COM2Port.Write(boutStop, 0, boutStop.Length);
                Thread.Sleep(50);
                //Dumby read to clear buffer of ACK from
                COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                Scan();
                Thread.Sleep(1000);
                myGreenLED.Write(false);
                mySPIPort.Write(myLCDDriver.LCDClear());
                Thread.Sleep(6);
                mySPIPort.Write(myLCDDriver.LCDLine1("Moving"));
                Thread.Sleep(6);

            }

            //Read the IR sensor. If the resulting value is greater
            //then the sensitivity value return true otherwise return false
            public bool SensorIR()
            {
                System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
                boutSensor = Encoding.GetBytes(sSensor);

                COM2Port.Write(boutSensor, 0, boutSensor.Length);
                Thread.Sleep(50);
                COM2Port.Read(binSensorArray, 0, binSensorArray.Length);

                int finalNum = StringToInt(binSensorArray);

                mySPIPort.Write(myLCDDriver.LCDClear());
                Thread.Sleep(6);
                mySPIPort.Write(myLCDDriver.LCDLine1("IR Value: " + finalNum.ToString()));

                //The sensitivity value can be changed for better sensitivity.
                if (finalNum > 135)
                {
                    return true;
                }
                else
                {
                    return false;
                }

            }

            //The Scan() method determines which direction the robot should turn.
            //3 IR readings are made - left, center, right. Based on the readings
            //The robot will make a turn to avoid the obstruction.
            public void Scan()
            {
                System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
                boutForward = Encoding.GetBytes(sForward);
                boutReverse = Encoding.GetBytes(sReverse);
                boutRight = Encoding.GetBytes(sRight);
                boutLeft = Encoding.GetBytes(sLeft);
                boutStop = Encoding.GetBytes(sSTOP);
                boutSensor = Encoding.GetBytes(sSensor);
                boutServo_m45 = Encoding.GetBytes(sServo_m45);
                boutServo_C = Encoding.GetBytes(sServo_C);
                boutServo_p45 = Encoding.GetBytes(sServo_p45);

                bool ScanLeft = false;
                bool ScanCenter = false;
                bool ScanRight = false;

                TempRead();

                mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Left"));
                COM2Port.Write(boutServo_m45, 0, boutServo_m45.Length);
                Thread.Sleep(500);
                //Dumby read to clear buffer of ACK from
                COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                if (SensorIR())
                {
                    ScanLeft = true;
                }

                mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Center"));
                COM2Port.Write(boutServo_C, 0, boutServo_C.Length);
                Thread.Sleep(500);
                //Dumby read to clear buffer of ACK from
                COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                if (SensorIR())
                {
                    ScanCenter = true;
                }


                mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Right"));
                COM2Port.Write(boutServo_p45, 0, boutServo_p45.Length);
                Thread.Sleep(500);
                //Dumby read to clear buffer of ACK from
                COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                if (SensorIR())
                {
                    ScanRight = true;
                }

                if (ScanLeft == true && ScanCenter == true && ScanRight == true)
                {
                    
                    //Obstruction ahead - turn around
                    COM2Port.Write(boutReverse, 0, boutReverse.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    Thread.Sleep(500);

                    COM2Port.Write(boutStop, 0, boutStop.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    ReverseCheck += 1;
                    Reversechecking();
                    Scan();


                }
                else if (ScanLeft == true && ScanCenter == true && ScanRight == false)
                {
                    
                    //turn right
                    COM2Port.Write(boutRight, 0, boutRight.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    Thread.Sleep(1000);

                    COM2Port.Write(boutStop, 0, boutStop.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);

                }
                else if (ScanLeft == true && ScanCenter == false && ScanRight == true)
                {

                    //Obstruction ahead - reverse
                    COM2Port.Write(boutReverse, 0, boutReverse.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    Thread.Sleep(500);

                    COM2Port.Write(boutStop, 0, boutStop.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    ReverseCheck += 1;
                    Reversechecking();
                    Scan();

                }
                else if (ScanLeft == false && ScanCenter == true && ScanRight == true)
                {
                    //Turn Left
                    COM2Port.Write(boutLeft, 0, boutLeft.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    Thread.Sleep(1000);

                    COM2Port.Write(boutStop, 0, boutStop.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);

                }
                else if (ScanLeft == false && ScanCenter == false && ScanRight == true)
                {
                    
                    //Turn Left
                    COM2Port.Write(boutLeft, 0, boutLeft.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    Thread.Sleep(500);

                    COM2Port.Write(boutStop, 0, boutStop.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);

                }
                else if (ScanLeft == true && ScanCenter == false && ScanRight == false)
                {
                    //turn right
                    COM2Port.Write(boutRight, 0, boutRight.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    Thread.Sleep(500);

                    COM2Port.Write(boutStop, 0, boutStop.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);

                }
                else if (ScanLeft == false && ScanCenter == false && ScanRight == false)
                {
                    COM2Port.Write(boutForward, 0, boutForward.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    Thread.Sleep(500);

                    COM2Port.Write(boutStop, 0, boutStop.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                }
                else if (ScanLeft == false && ScanCenter == true && ScanRight == false)
                {
                    COM2Port.Write(boutReverse, 0, boutReverse.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    Thread.Sleep(500);

                    COM2Port.Write(boutStop, 0, boutStop.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    ReverseCheck += 1;
                    Reversechecking();
                    Scan();
                }




            }
            //-------------------------------------------------------------------------------------------
            // Converts Strings to an Integer values
            // Used for both IR and Temp Readings
            // Assumption: number field in byte string has ASCII space, 0x20, after it: 
            // <preampble characters><ASCII digits><ASCII space><trailer characters>
            // Number is always positive
            //-------------------------------------------------------------------------------------------

            public int StringToInt(byte[] AsciiString)
            {
                int iReturnVal = 0;
                int idx;
                int iPwrTen;   //Conversion multiplier

                for (idx = 0; AsciiString[idx] != 0x20; idx++)
                {
                    ;   //Do nothing
                }

                idx--;  //Back up to units digit
                for (iPwrTen = 1; ((AsciiString[idx] >= 0x30) && (AsciiString[idx] <= 0x39)); idx--)
                {
                    iReturnVal += (AsciiString[idx] & 0x0F) * iPwrTen;  //Mask off upper ASCII nibble and convert to power of ten
                    iPwrTen *= 10;
                }

                return iReturnVal;
            }

            //Sends Temperature Reading to the LCD display
            public void TempRead()
            {

                Thread.Sleep(500);

                System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
                boutTempSensor = Encoding.GetBytes(tempSensor);

                COM2Port.Write(boutTempSensor, 0, boutTempSensor.Length);
                Thread.Sleep(50);
                COM2Port.Read(binSensorArray, 0, binSensorArray.Length);

                int finalNum = StringToInt(binSensorArray);

                float tempC = ((((float)finalNum * 5) / 1024) * 100) - 273;
                float tempF = (tempC * (float)1.8) + 32;

                mySPIPort.Write(myLCDDriver.LCDLine1("Temp: " + tempF.ToString() + "F / " + tempC.ToString() + "C"));

                Thread.Sleep(1000);


            }


            //If we backed up more than once and spin around to avoid
            //being stuck. The global ReverseCheck int is used
            public void Reversechecking()
            {

                System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
                boutRight = Encoding.GetBytes(sRight);
                boutLeft = Encoding.GetBytes(sLeft);
                boutStop = Encoding.GetBytes(sSTOP);

                if (ReverseCheck >= 2)
                {

                    //Spin around
                    COM2Port.Write(boutRight, 0, boutRight.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    Thread.Sleep(1800);

                    COM2Port.Write(boutStop, 0, boutStop.Length);
                    Thread.Sleep(50);
                    //Dumby read to clear buffer of ACK from
                    COM2Port.Read(binSensorArray, 0, binSensorArray.Length);
                    ReverseCheck = 0;

                }


            }
            

        }
        

    }
}
